
/* Network traffic analyzer
 * (c) 2001, Emil Mikulic.
 */

/* Protocol database */

#include "proto_db.h"



/* Global database */
db proto_db;



void proto_db_init(void)
{
	db_init(&proto_db);
}

void proto_db_free(void)
{
	unsigned int i;

	for (i=0; i<proto_db.used; i++)
		free(proto_db.records[i]);

	db_free(&proto_db);
}



/* proto_record needs to be deallocated when it's no longer needed */
proto_record *proto_make(const byte protocol)
{
	proto_record *p = (proto_record*)malloc(sizeof(proto_record));

	p->protocol = protocol;
	p->data_in.hi = p->data_in.lo =
		p->data_out.hi = p->data_out.lo = 0;

	return p;
}



/* update transfer amount */
__inline void proto_update_in(proto_record *p, const dword in)
{
	i64 add = {0, in};
	add_64bit(&(p->data_in), add);
}

__inline void proto_update_out(proto_record *p, const dword out)
{
	i64 add = {0, out};
	add_64bit(&(p->data_out), add);
}



/* add or update a protocol, depending on whether it exists in the db or not */
__inline proto_record *proto_from_num(const byte proto)
{
	int i;
	proto_record *p;
	
	p = NULL;
	for (i=0; i<proto_db.used; i++)
	{
		proto_record *tmp = (proto_record*)db_get(proto_db,i);
		if (tmp->protocol == proto) p = tmp;
	}

	if ((proto_db.used == 0) || (p == NULL))
	{
		p = proto_make(proto);
		if (!db_add(&proto_db, p)) freakout();
	}

	return p;
}


__inline void proto_transfer_in(const byte proto, const dword in)
{
	proto_update_in(proto_from_num(proto), in);
}

__inline void proto_transfer_out(const byte proto, const dword out)
{
	proto_update_out(proto_from_num(proto), out);
}



void proto_db_dump(FILE *fp)
{
	int i;
	
	fprintf(fp, "%u\n", proto_db.used); /* can't be more than 256 */
	for (i=0; i<proto_db.used; i++)
	{
		proto_record *p = (proto_record*)db_get(proto_db, i);
		
		fprintf(fp, "%u|%u:%u|%u:%u\n",
			p->protocol,
			p->data_in.hi,
			p->data_in.lo,
			p->data_out.hi,
			p->data_out.lo);
	}
}



char *proto_name(const byte protocol)
{
	/* full list in: netinet/in.h */
	switch (protocol) {
		case 0: return "TCP Dummy protocol";
		case 1: return "Internet Control Message Protocol";
		case 2: return "Internet Group Management Protocol";
		case 4: return "IPIP Tunnel";
		case 94: return "IPIP Tunnel (older KA9Q variety)";
		case 6: return "Transmission Control Protocol";
		case 8: return "Exterior Gateway Protocol";
		case 12: return "PUP Protocol";
		case 17: return "User Datagram Protocol";
		case 22: return "XNS IDP Protocol";
		case 29: return "SO Transport Protocol Class 4";
		case 41: return "IPv6 header";
		case 43: return "IPv6 routing header";
		case 44: return "IPv6 fragmentation header";
		case 46: return "Reservation Protocol";
		case 47: return "General Routing Encapsulation";
		case 50: return "Encapsulating Security Payload";
		case 51: return "Authentication Header";
		case 58: return "ICMPv6";
		case 59: return "IPv6 no next header";
		case 60: return "IPv6 destination options";
		case 92: return "Multicast Transport Protocol";
		case 98: return "Encapsulation Header";
		case 103: return "Protocol Independent Multicast";
		case 108: return "Compression Header Protocol";
		case 255: return "Raw IP packet";
	}
	return "&nbsp;";
}



char *proto_name_short(const byte protocol)
{
	/* full list in: netinet/in.h */
	switch (protocol) {
		case 0: return "Dummy";
		case 1: return "ICMP";
		case 2: return "IGMP";
		case 4: return "IPIP";
		case 94: return "IPIP-KA9Q";
		case 6: return "TCP";
		case 8: return "EGP";
		case 12: return "PUP";
		case 17: return "UDP";
		case 22: return "XNS-IDP";
		case 58: return "ICMPv6";
		case 103: return "PImcast";
		case 255: return "Raw";
	}
	return "UNKNOWN";
}


